/*
 * Copyright (C) 2003 John David Ratliff
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package gui.view;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;

import javax.swing.text.Document;
import javax.swing.text.EditorKit;
import javax.swing.text.StyledEditorKit;

import com.ozten.font.JFontChooser;

import de.hunsicker.jalopy.Jalopy;
import de.hunsicker.jalopy.swing.SettingsDialog;

import gui.model.SyntaxDocument;

/**
 * A simple GUI application to format Java files using Jalopy.
 *
 * @author John David Ratliff
 * @version 1.6, 09/27/03
 */
public class JalopyFrame extends JFrame {
    private JTextPane editor;
    private JMenuItem editFromClipboard;
    private JMenuItem fileOpen;
    private JMenu editMenu;
    private JMenuItem fileSave;
    private JMenuItem fileSaveAs;
    private JScrollPane editorScroller;
    private JMenuItem editToClipboard;
    private JMenuItem editSetFont;
    private JMenuItem fileExit;
    private JMenu fileMenu;
    private JButton formatButton;
    private JMenuItem helpAbout;
    private JMenu helpMenu;
    private JMenuItem jalopyFormat;
    private JMenuItem jalopySettings;
    private JMenu jalopyMenu;
    private JMenuBar mainMenu;
    private File file = new File("Default.java");
    private boolean isFile;
    private boolean modified;
    private static final JFileChooser FILE_CHOOSER = buildFileChooser();

    /**
     * Creates a new JalopyFrame object.
     */
    public JalopyFrame() {
        initComponents();
    }

    /**
     * Builds the Java Source File chooser.
     *
     * @return The JFileChooser for selecting Java source files.
     */
    private static JFileChooser buildFileChooser() {
        JFileChooser chooser = new JFileChooser();
        JavaFileFilter filter = new JavaFileFilter();

        chooser.setFileFilter(filter);

        return chooser;
    }

    /**
     * Initializes the frame components.
     */
    private void initComponents() {
        GridBagConstraints gridBagConstraints;

        editorScroller = new JScrollPane();
        editor = new JTextPane();
        formatButton = new JButton();
        mainMenu = new JMenuBar();
        fileMenu = new JMenu();
        fileExit = new JMenuItem();
        editMenu = new JMenu();
        fileOpen = new JMenuItem();
        fileSave = new JMenuItem();
        fileSaveAs = new JMenuItem();
        editFromClipboard = new JMenuItem();
        editToClipboard = new JMenuItem();
        editSetFont = new JMenuItem();
        jalopyMenu = new JMenu();
        jalopyFormat = new JMenuItem();
        jalopySettings = new JMenuItem();
        helpMenu = new JMenu();
        helpAbout = new JMenuItem();

        getContentPane().setLayout(new GridBagLayout());

        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        setTitle("Jalopy Source Formatter");
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent evt) {
                    fileExitAction();
                }
            });

        // setup the editor properties
        editor.setEditable(false);
        editor.setFont(new Font("Courier", Font.PLAIN, 12));

        EditorKit kit = new StyledEditorKit() {
            public Document createDefaultDocument() {
                return new SyntaxDocument();
            }
        };

        editor.setEditorKitForContentType("text/java", kit);
        editor.setContentType("text/java");

        editorScroller.setViewportView(editor);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(editorScroller, gridBagConstraints);

        formatButton.setMnemonic('o');
        formatButton.setText("Format");
        formatButton.setToolTipText("Format the editor buffer with Jalopy...");

        formatButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    jalopyFormatAction();
                }
            });

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new Insets(0, 0, 5, 0);
        getContentPane().add(formatButton, gridBagConstraints);

        fileMenu.setMnemonic('f');
        fileMenu.setText("File");
        fileOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
                                                       InputEvent.SHIFT_MASK |
                                                       InputEvent.CTRL_MASK));
        fileOpen.setMnemonic('o');
        fileOpen.setText("Open");
        fileOpen.setToolTipText("Open a file in the editor...");

        fileOpen.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    fileOpenAction();
                }
            });
        fileMenu.add(fileOpen);

        fileSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
                                                       InputEvent.SHIFT_MASK |
                                                       InputEvent.CTRL_MASK));
        fileSave.setMnemonic('s');
        fileSave.setText("Save");
        fileSave.setToolTipText("Save the editor contents to a file...");

        fileSave.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    fileSaveAction();
                }
            });

        fileMenu.add(fileSave);

        fileSaveAs.setMnemonic('a');
        fileSaveAs.setText("Save As");
        fileSaveAs.setToolTipText("Save the editor contents to a file...");

        fileSaveAs.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    fileSaveAsAction();
                }
            });

        fileMenu.add(fileSaveAs);

        fileMenu.add(new JSeparator());

        fileExit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
                                                       InputEvent.SHIFT_MASK |
                                                       InputEvent.CTRL_MASK));
        fileExit.setMnemonic('x');
        fileExit.setText("Exit");
        fileExit.setToolTipText("Exits the application...");

        fileExit.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    fileExitAction();
                }
            });

        fileMenu.add(fileExit);

        mainMenu.add(fileMenu);

        editMenu.setMnemonic('e');
        editMenu.setText("Edit");

        editToClipboard.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,
                                                              InputEvent.SHIFT_MASK |
                                                              InputEvent.CTRL_MASK));
        editToClipboard.setMnemonic('c');
        editToClipboard.setText("Copy");
        editToClipboard.setToolTipText("Copies the editor contents to the clipboard...");

        editToClipboard.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    editToClipboardAction();
                }
            });

        editMenu.add(editToClipboard);

        editFromClipboard.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,
                                                                InputEvent.SHIFT_MASK |
                                                                InputEvent.CTRL_MASK));
        editFromClipboard.setMnemonic('p');
        editFromClipboard.setText("Paste");
        editFromClipboard.setToolTipText("Copies the contents of the clipboard to the editor...");

        editFromClipboard.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    editFromClipboardAction();
                }
            });

        editMenu.add(editFromClipboard);
        
        editMenu.add(new JSeparator());

        editSetFont.setMnemonic('f');
        editSetFont.setText("Set Font");
        editSetFont.setToolTipText("Changes the font used by the editor...");

        editSetFont.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    editSetFontAction();
                }
            });

        editMenu.add(editSetFont);

        mainMenu.add(editMenu);

        jalopyMenu.setMnemonic('j');
        jalopyMenu.setText("Jalopy");
        jalopyFormat.setMnemonic('f');
        jalopyFormat.setText("Format");
        jalopyFormat.setToolTipText("Format the editor contents using Jalopy...");

        jalopyFormat.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    jalopyFormatAction();
                }
            });

        jalopyMenu.add(jalopyFormat);

        jalopySettings.setMnemonic('s');
        jalopySettings.setText("Settings");
        jalopySettings.setToolTipText("Modifies the current Jalopy settings...");

        jalopySettings.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    jalopySettingsAction();
                }
            });

        jalopyMenu.add(jalopySettings);

        mainMenu.add(jalopyMenu);

        helpMenu.setMnemonic('h');
        helpMenu.setText("Help");
        helpAbout.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,
                                                        InputEvent.SHIFT_MASK |
                                                        InputEvent.CTRL_MASK));
        helpAbout.setMnemonic('a');
        helpAbout.setText("About");
        helpAbout.setToolTipText("Displays about information...");

        helpAbout.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    helpAboutAction();
                }
            });

        helpMenu.add(helpAbout);

        mainMenu.add(helpMenu);

        setJMenuBar(mainMenu);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width - 640) / 2,
                  (screenSize.height - 480) / 2, 640, 480);
    }

    /**
     * The action performed when open is selected from the file menu.
     */
    private void fileOpenAction() {
        if (FILE_CHOOSER.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            // open the selected file
            File file = FILE_CHOOSER.getSelectedFile();

            try {
                FileInputStream input = new FileInputStream(file);

                this.editor.read(input, null);
                input.close();

                this.file = file;
                this.isFile = true;
            } catch (FileNotFoundException e) {
                // this will never happen...
                e.printStackTrace();
            } catch (IOException e) {
                // this shouldn't happen...
                e.printStackTrace();
            }
        }
    }

    /**
     * Serializes the editor contents to file.
     *
     * @param file The file to serialize the contents to.
     */
    private void serializeBuffer(File file) {
        try {
            PrintWriter writer = new PrintWriter(new FileOutputStream(file));

            writer.println(this.editor.getText());
            writer.close();

            this.modified = false;
            this.file = file;
        } catch (FileNotFoundException e) {
            // this shouldn't matter...
            e.printStackTrace();
        }
    }

    /**
     * The action performed when save is selected from the file menu.
     */
    private void fileSaveAction() {
        if (this.isFile) {
            this.serializeBuffer(this.file);
        } else {
            this.fileSaveAsAction();
        }
    }

    /**
     * The action performed when save as is selected from the file menu.
     */
    private void fileSaveAsAction() {
        if (FILE_CHOOSER.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
            // save the contents to the selected file
            File file = FILE_CHOOSER.getSelectedFile();

            if (file.exists()) {
                int result = JOptionPane.showConfirmDialog(this,
                                                           "File Already Exists.\nOverwrite " +
                                                           file.getName() +
                                                           "?",
                                                           "Confirm File Overwrite",
                                                           JOptionPane.YES_NO_OPTION);

                if (result == JOptionPane.NO_OPTION) {
                    return;
                }
            }

            // serialize the buffer
            this.serializeBuffer(file);
        }
    }

    /**
     * The action performed when exit is selected from the file menu.
     */
    private void fileExitAction() {
        if (this.modified) {
            int result = JOptionPane.showConfirmDialog(this,
                                                       "The editor has been formatted, but not saved or copied to clipboard.\nDo you really want to exit?",
                                                       "Editor Contents Unsaved",
                                                       JOptionPane.YES_NO_OPTION);

            if (result == JOptionPane.NO_OPTION) {
                return;
            }
        }

        System.exit(0);
    }

    /**
     * The action performed when paste is selected from the edit menu.
     */
    private void editFromClipboardAction() {
        // must be editable to paste
        this.editor.setEditable(true);

        // paste the selection
        this.editor.selectAll();
        this.editor.paste();

        // return to non-editable state
        this.editor.setEditable(false);

        // no longer working from a File
        this.isFile = false;
    }

    /**
     * The action performed when copy to is selected from the edit menu.
     */
    private void editToClipboardAction() {
        this.editor.selectAll();
        this.editor.copy();
        this.modified = false;
    }

    /**
     * The action performed when set font is selected from the edit menu.
     */
    private void editSetFontAction() {
        Font font = JFontChooser.showDialog(this, "Font Selection",
                                            "The quick brown fox jumped over the lazy dog.",
                                            this.editor.getFont());
        this.editor.setFont(font);
    }

    /**
     * The action performed when format is selected from the jalopy menu.
     */
    private void jalopyFormatAction() {
        String input = this.editor.getText();
        StringBuffer output = new StringBuffer();
        Jalopy jalopy = new Jalopy();

        // setup the Jalopy input/output
        jalopy.setInput(input, this.file.getName());
        jalopy.setOutput(output);

        // format the code using Jalopy
        jalopy.format();

        // check the status of the format
        if (jalopy.getState() == Jalopy.State.ERROR) {
            // errors. this is BAD
            JOptionPane.showMessageDialog(this,
                                          "Unable to format this code with Jalopy",
                                          "Error: Jalopy cannot process this code",
                                          JOptionPane.ERROR_MESSAGE);
        } else {
            this.editor.setText(output.toString());
            this.modified = true;
        }
    }

    /**
     * The action performed when settings is selected from the jalopy menu.
     */
    private void jalopySettingsAction() {
        SettingsDialog dialog = SettingsDialog.create(this);

        dialog.pack();
        dialog.setVisible(true);
    }

    /**
     * The action performed when about is selected from the help menu.
     */
    private void helpAboutAction() {
        JOptionPane.showMessageDialog(this,
                                      "Jalopy Formatter GUI Version 1.0.6\nCopyright (C) 2003 John David Ratliff",
                                      "About Jalopy Formatter GUI...",
                                      JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * Starts the application.
     *
     * @param args not used.
     */
    public static void main(String[] args) {
        new JalopyFrame().show();
    }
}
